<?php declare(strict_types=1);

namespace Infomaker\Everyware\NewsML\Parsers;

use Infomaker\Everyware\Support\Arr;
use Infomaker\Everyware\NewsML\Item;
use Infomaker\Everyware\NewsML\TypeParser;
use SimpleXMLIterator;

/**
 * ContentPartParser
 *
 * @link    http://infomaker.se
 * @package Infomaker\Everyware\NewsML\Parsers
 * @since   Infomaker\Everyware\NewsML\Parsers\ContentPartParser 1.0.0
 */
class ContentPartParser implements ItemParser, TypeParser
{

    public const OBJECT_TYPE = 'x-im/content-part';

    /**
     * @var LinkParser
     */
    protected $linkParser;

    /**
     * @var ItemParser[]
     */
    private $textParsers;

    public function __construct(ElementParser $parser)
    {

        $this->textParsers = [
            ElementParser::TAG => $parser
        ];

        $this->linkParser = new LinkParser();
    }

    public function parse(SimpleXMLIterator $element): array
    {
        $item = new Item($element);

        $item->fill(array_replace($this->getData($element), [
            'type' => $item->getType(),
            'content_type' => $this->getContentType($element)
        ]));

        return [$item];
    }

    protected function getData(SimpleXMLIterator $element): array
    {
        $defaultData = [
            'title' => '',
            'text' => [],
            'subject' => ''
        ];

        if (!property_exists($element, 'data')) {
            return $defaultData;
        }

        $data = $element->data->children();

        foreach ($data as $tag => $item) {
            $defaultData[$tag] = $tag === 'text' ? $this->parseText($item->children()) : (string)$item;
        }

        return $defaultData;
    }

    private function parseText(SimpleXMLIterator $textIterator): array
    {
        $items = [];
        foreach ($textIterator as $tag => $item) {
            $parser = Arr::get($this->textParsers, $tag, null);
            $items[] = $parser instanceof ItemParser ? $parser->parse($item) : new Item($item);
        }

        return Arr::flatten($items, 1);
    }

    private function getContentType(SimpleXMLIterator $element): string
    {
        $links = $this->linkParser->parseLinks($element);
        if (!empty($links)) {
            foreach ($links as $index => $link) {
                if ($link instanceof Item) {
                    $attributes = $link->getItemAttributes();
                    if (Arr::get($attributes, 'rel', '') === 'content-part') {
                        return str_replace('im:/content-part/', '', Arr::get($attributes, 'uri', ''));
                    }
                }
            }
        }

        return '';
    }

    public function typeToParse(): string
    {
        return static::OBJECT_TYPE;
    }
}
